<?php
/*
Plugin Name: navigo 
Plugin URI: http://adahas.com/work/navigo/
Description: 'navigo' allows you to display custom navigation menus (for applying different styles and highlighting) for pages. Configuration options can be found under 'Options' in wp-admin. This has been tested and works fine on WordPress 1.5+
Author: Venushka Perera
Version: 1.1
Author URI: http://adahas.com/
*/ 

/*  
Copyright 2005  Venushka Perera  (email : venushka@adahas.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License Version 2, as 
published by the Free Software Foundation in June 1991.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program (navigo-license-gpl.txt); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, 
	Suite 330, 
	Boston, 
	MA 02111-1307 USA*/

/*
Do not modify the following code to manipulate the output of this plugin. 
For configuration options, please see Options in wp-admin.
*/

load_plugin_textdomain('navigo');

function navigo($args = '', $from_page_content = false) {
	$content = '';

	global $wp_query;
	$queried_obj = $wp_query->get_queried_object();

	// parse all args, and if not specified, initialize to defaults
	parse_str($args, $r);
	
	if (!isset($r['child_of'])) $r['child_of'] = 0;
	if (!isset($r['depth'])) $r['depth'] = 0;
	if (!isset($r['hidden_depth'])) $r['hidden_depth'] = 0;
	if (!isset($r['collapse'])) $r['collapse'] = false;
	if (!isset($r['leading_tabs'])) $r['leading_tabs'] = 2;
	if (!isset($r['sort_column'])) $r['sort_column'] = '';
	
	// excludes single menu points
	if (!isset($r['exclude'])) { $r['exclude'] = ''; } 
		else { $r['exclude'] = preg_split('/[\s,]+/',$r['exclude']); }	
		
	// get all page information
	$pages_params = '';
	if(strlen($r['sort_column']) > 0)
		$pages_params = 'sort_column=' . $r['sort_column'];
	
	$pages = get_pages($pages_params);
	
	if ( $pages ) {
			
	
		// construct the page tree
		$page_tree = array();

		// collect all page ids
		$keys = array_keys($pages);
		$indexes = array();
		for($i = 0; $i < sizeof($keys); $i++) {
			$indexes[$keys[$i]] = $pages[$keys[$i]]->ID;
		}

		// collect all parent ids
		$parents = array();
		$child_id = $queried_obj->ID;
		
		while($child_id != 0) {
			$parent_id = $pages[array_search($child_id, $indexes)]->post_parent;
			array_push($parents, $parent_id);
			$child_id = $parent_id;
		}
		
		// clear parent if root differs from the current page root
		if($r['child_of'] != 0 && !strcmp(array_search($r['child_of'], $parents), '') ) {
			$parents = array();
		}
		
		foreach($pages as $page) {
		$excludepage = false;
		if ( !empty($r['exclude']) ) {
		if ( in_array($page->ID, $r['exclude'], true) )
		$excludepage = true;
		}
		if ( $excludepage == false ) {
		$page_tree[$page->ID]['title'] = $page->post_title;
		$page_tree[$page->ID]['name'] = $page->post_name;
		$page_tree[$page->post_parent]['children'][] = $page->ID;
		}
		}
		
		// prepare indentation level
		$indent = "";
		for($i = 0; $i < ($r['leading_tabs'] - $r['hidden_depth']); $i++) {
			$indent .= "\t";
		}

		// root list element - start
		if($r['hidden_depth'] < 1) {
			$id = 'level1';
			if(isset($r[$id])) $id = $r[$id];
			
			$attribute = "id";
			if($r['collapse'])
				$attribute = "id";
			
			$content .= "\n" . $indent . '<ul ' . $attribute . '="' . $id . '">';
		}
		
		// method call to display level
		$content .= _navigo_subs($r['child_of'], $page_tree, $r, $parents, $pages, $indexes, $queried_obj, 1);
		
		// root list element - end
		if($r['hidden_depth'] < 1) {
			$content .= "\n" . $indent . '</ul>';
		}
	}
	

	$content .= "" . '<!-- navigation list code generated by \'navigo\' - http://adahas.com/work/navigo/ -->' . "\n";
	if($from_page_content)
		return $content;
	else
		echo($content);
}

function _navigo_subs($parent, $page_tree, $r, $parents, $pages, $indexes, $queried_obj, $current_level) {

	$content = '';

	foreach($page_tree[$parent]['children'] as $page_id) {
	
		// get a reference to the current page for easy access
		$cur_page = $page_tree[$page_id];
		
		// prepare indentation level
		$indent = "";
		for($i = 0; $i < ($r['leading_tabs'] + $current_level - $r['hidden_depth']); $i++) {
			$indent .= "\t";
		}
		
		// prepare current and active ids
		$active = '';
		$current = '';
		
		$attribute = "class";
		if($r['collapse'])
			$attribute = "id";
		
		// prepare ids for active and current elements
		if(in_array($page_id, $parents) || $page_id == $queried_obj->ID) {
			$id = 'level' . $current_level;
			if(isset($r[$id])) $id = $r[$id];
		
			$active = ' ' . $attribute . '="' . $id . 'active"';
		
			if(in_array($page_id, $parents)) {
				$current = ' ' . $attribute . '="' . $id . 'highlight"';
			}
			else {
				$current = ' ' . $attribute . '="' . $id . 'current"';
			}
		}

		// start list item element if hidden depth not exceeded
		if( $r['hidden_depth'] < $current_level)  {
			$content .= "\n" . $indent . '<li' . $active . '>';
			$content .= '<a' . $current . ' href="' . get_page_link($page_id) . '" title="' . wp_specialchars($cur_page['title']) . '">' . $cur_page['title'] . '</a>';
		}
		
		// restrict dispaying if collapsable mode
		if((! $r['collapse']) || (in_array($page_id, $parents) || $page_id == $queried_obj->ID)) {
		
			// check whether element is a leaf node
			if(isset($cur_page['children']) && is_array($cur_page['children'])) {
			
				// check for display depth limit
				if($r['depth'] > $current_level || $r['depth'] == 0) {
				
					// sub list element - start
					$id = 'level' . ($current_level + 1);
					if(isset($r[$id])) $id = $r[$id];
					
					$content .= "\n" . $indent . '<ul ' . $attribute . '="' . $id . '">';
					
					// method call to display level
					$content .= _navigo_subs($page_id, $page_tree, $r, $parents, $pages, $indexes, $queried_obj, $current_level + 1);
					
					// sub list element - start
					$content .= "\n" . $indent . '</ul>';
				}
			}
		}

		// end list item elements if hidden depth not exceeded
		if( $r['hidden_depth'] < $current_level)  {
			$content .= '</li>';
		}
	}

	return $content;
}

// replace placeholder by generated code
function navigo_insert( $content ) {
	$matches = array();
	if(preg_match('#<!--navigo:(.*)-->#', $content, $matches)) {
		$replace_string = '<!--navigo:' . $matches[1] . '-->';
		$matches[1] = str_replace(', ', '&', $matches[1]);
		$content = str_replace($replace_string, navigo(trim($matches[1]), true), $content);
	}
	return $content;
}

// display options tab in wp-admin
function navigo_options_page() {
	add_options_page('navigo Options', 'navigo', 9, 'navigo-options.php');
}

// add actions and filters
add_action('admin_head', 'navigo_options_page');
add_filter('the_content', 'navigo_insert');
?>